@@ -840,15 +840,16 @@ long cm_EnumerateCellRegistry(afs_uint32 client, cm_enumCellProc_t *procp, void
     LONG code;
     FILETIME ftLastWriteTime;
     char szCellName[CELL_MAXNAMELEN];
+	char * subkey = AFSREG_CLT_OPENAFS_SUBKEY "\\CellServDB";
 
     /* No server CellServDB in the registry. */
     if (!client || procp == NULL)
         return 0;
 
     if (RegOpenKeyEx( HKEY_LOCAL_MACHINE,
-                      AFSREG_CLT_OPENAFS_SUBKEY "\\CellServDB",
+		      subkey,
                       0,
-                      KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
+		      KEY_READ|KEY_QUERY_VALUE,
                       &hkCellServDB) != ERROR_SUCCESS)
         return 0;
 
@@ -918,7 +919,7 @@ long cm_SearchCellByDNS(char *cellNamep, char *newCellNamep, int *ttl,
     rc = getAFSServer("afs3-vlserver", "udp", cellNamep, htons(7003),
                       cellHostAddrs, cellHostNames, ports, adminRanks, &numServers, ttl);
     if (rc == 0 && numServers > 0) {     /* found the cell */
-        for (i = 0; i < numServers; i++) {
+	for (i = 0; i < numServers; i++) {
             memcpy(&vlSockAddr.sin_addr.s_addr, &cellHostAddrs[i],
                    sizeof(long));
             vlSockAddr.sin_port = ports[i];
