@@ -11,9 +11,6 @@
 #include "afs/param.h"
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
-#ifndef AFS_LINUX22_ENV
-#include "rpc/types.h"
-#endif
 #ifdef	AFS_ALPHA_ENV
 #undef kmem_alloc
 #undef kmem_free
@@ -240,18 +237,15 @@ rxfs_storeClose(void *r, struct AFSFetchStatus *OutStatus, int *doProcessFS)
 }
 
 afs_int32
-rxfs_storeDestroy(void **r, afs_int32 error)
+rxfs_storeDestroy(void **r, afs_int32 code)
 {
-    afs_int32 code = error;
     struct rxfs_storeVariables *v = (struct rxfs_storeVariables *)*r;
 
     *r = NULL;
     if (v->call) {
 	RX_AFS_GUNLOCK();
-	code = rx_EndCall(v->call, error);
+	code = rx_EndCall(v->call, code);
 	RX_AFS_GLOCK();
-	if (!code && error)
-	    code = error;
     }
     if (v->tbuffer)
 	osi_FreeLargeSpace(v->tbuffer);
@@ -276,6 +270,7 @@ afs_GenericStoreProc(struct storeOps *ops, void *rock,
     size = tdc->f.chunkBytes;
 
     tfile = afs_CFileOpen(&tdc->f.inode);
+    osi_Assert(tfile);
 
     while ( size > 0 ) {
 	code = (*ops->prepare)(rock, size, &tlen);
@@ -329,12 +324,8 @@ struct storeOps rxfs_storeUfsOps = {
     .padd =	rxfs_storePadd,
     .close =	rxfs_storeClose,
     .destroy =	rxfs_storeDestroy,
-#ifdef AFS_LINUX26_ENV
-    .storeproc = afs_linux_storeproc
-#else
     .storeproc = afs_GenericStoreProc
 #endif
-#endif
 };
 
 static
@@ -465,6 +456,7 @@ afs_CacheStoreDCaches(struct vcache *avc, struct dcache **dclist,
 {
     int *shouldwake = NULL;
     unsigned int i;
+    int stored = 0;
     afs_int32 code = 0;
     afs_size_t bytesXferred;
 
@@ -476,7 +468,6 @@ afs_CacheStoreDCaches(struct vcache *avc, struct dcache **dclist,
     osi_Assert(nchunks != 0);
 
     for (i = 0; i < nchunks && !code; i++) {
-	int stored = 0;
 	struct dcache *tdc = dclist[i];
 	afs_int32 size;
 
@@ -811,7 +802,7 @@ afs_int32
 rxfs_fetchClose(void *r, struct vcache *avc, struct dcache * adc,
 		struct afs_FetchOutput *o)
 {
-    afs_int32 code, code1 = 0;
+    afs_int32 code;
     struct rxfs_fetchVariables *v = (struct rxfs_fetchVariables *)r;
 
     if (!v->call)
@@ -826,10 +817,8 @@ rxfs_fetchClose(void *r, struct vcache *avc, struct dcache * adc,
 #endif
         code = EndRXAFS_FetchData(v->call, &o->OutStatus, &o->CallBack,
 				&o->tsync);
-    code1 = rx_EndCall(v->call, code);
+    code = rx_EndCall(v->call, code);
     RX_AFS_GLOCK();
-    if (!code && code1)
-	code = code1;
 
     v->call = NULL;
 
@@ -837,18 +826,15 @@ rxfs_fetchClose(void *r, struct vcache *avc, struct dcache * adc,
 }
 
 afs_int32
-rxfs_fetchDestroy(void **r, afs_int32 error)
+rxfs_fetchDestroy(void **r, afs_int32 code)
 {
-    afs_int32 code = error;
     struct rxfs_fetchVariables *v = (struct rxfs_fetchVariables *)*r;
 
     *r = NULL;
     if (v->call) {
         RX_AFS_GUNLOCK();
-	code = rx_EndCall(v->call, error);
+	code = rx_EndCall(v->call, code);
         RX_AFS_GLOCK();
-	if (error)
-	    code = error;
     }
     if (v->tbuffer)
 	osi_FreeLargeSpace(v->tbuffer);
@@ -914,11 +900,11 @@ rxfs_fetchInit(struct afs_conn *tc, struct rx_connection *rxconn,
 	       struct osi_file *fP, struct fetchOps **ops, void **rock)
 {
     struct rxfs_fetchVariables *v;
-    int code = 0, code1 = 0;
+    int code = 0;
 #ifdef AFS_64BIT_CLIENT
     afs_uint32 length_hi = 0;
 #endif
-    afs_uint32 length, bytes;
+    afs_uint32 length = 0, bytes;
 
     v = (struct rxfs_fetchVariables *)
 	    osi_AllocSmallSpace(sizeof(struct rxfs_fetchVariables));
@@ -948,9 +934,8 @@ rxfs_fetchInit(struct afs_conn *tc, struct rx_connection *rxconn,
 		if (bytes == sizeof(afs_int32)) {
 		    length_hi = ntohl(length_hi);
 		} else {
-		    code = rx_Error(v->call);
 		    RX_AFS_GUNLOCK();
-		    code1 = rx_EndCall(v->call, code);
+		    code = rx_EndCall(v->call, RX_PROTOCOL_ERROR);
 		    RX_AFS_GLOCK();
 		    v->call = NULL;
 		}
@@ -982,18 +967,55 @@ rxfs_fetchInit(struct afs_conn *tc, struct rx_connection *rxconn,
 		length = ntohl(length);
 	    else {
 		RX_AFS_GUNLOCK();
-		code = rx_Error(v->call);
-                code1 = rx_EndCall(v->call, code);
+                code = rx_EndCall(v->call, RX_PROTOCOL_ERROR);
 		v->call = NULL;
+		length = 0;
 		RX_AFS_GLOCK();
 	    }
 	}
 	FillInt64(length64, length_hi, length);
+
+        if (!code) {
+            /* Check if the fileserver said our length is bigger than can fit
+             * in a signed 32-bit integer. If it is, we can't handle that, so
+             * error out. */
+	    if (length64 > MAX_AFS_INT32) {
+                static int warned;
+                if (!warned) {
+                    warned = 1;
+                    afs_warn("afs: Warning: FetchData64 returned too much data "
+                             "(length64 %u.%u); this should not happen! "
+                             "Aborting fetch request.\n",
+                             length_hi, length);
+                }
+		RX_AFS_GUNLOCK();
+                code = rx_EndCall(v->call, RX_PROTOCOL_ERROR);
+		v->call = NULL;
+		length = 0;
+		RX_AFS_GLOCK();
+                code = code != 0 ? code : EIO;
+            }
+        }
+
+        if (!code) {
+            /* Check if the fileserver said our length was negative. If it
+             * is, just treat it as a 0 length, since some older fileservers
+             * returned negative numbers when they meant to return 0. Note
+             * that we must do this in this 64-bit-specific block, since
+             * length64 being negative will screw up our conversion to the
+             * 32-bit 'alength' below. */
+            if (length64 < 0) {
+                length_hi = length = 0;
+                FillInt64(length64, 0, 0);
+            }
+        }
+
 	afs_Trace3(afs_iclSetp, CM_TRACE_FETCH64LENG,
 		   ICL_TYPE_POINTER, avc, ICL_TYPE_INT32, code,
 		   ICL_TYPE_OFFSET,
 		   ICL_HANDLE_OFFSET(length64));
-	*alength = length;
+	if (!code)
+	    *alength = length;
 #else /* AFS_64BIT_CLIENT */
 	RX_AFS_GUNLOCK();
 	code = StartRXAFS_FetchData(v->call, (struct AFSFid *)&avc->f.fid.Fid,
@@ -1006,9 +1028,14 @@ rxfs_fetchInit(struct afs_conn *tc, struct rx_connection *rxconn,
 	    RX_AFS_GLOCK();
 	    if (bytes == sizeof(afs_int32)) {
                 *alength = ntohl(length);
+                if (*alength < 0) {
+                    /* Older fileservers can return a negative length when they
+                     * meant to return 0; just assume negative lengths were
+                     * meant to be 0 lengths. */
+                    *alength = 0;
+                }
 	    } else {
-		code = rx_Error(v->call);
-                code1 = rx_EndCall(v->call, code);
+                code = rx_EndCall(v->call, RX_PROTOCOL_ERROR);
 		v->call = NULL;
 	    }
 	}
@@ -1024,17 +1051,21 @@ rxfs_fetchInit(struct afs_conn *tc, struct rx_connection *rxconn,
 	 * requested. It shouldn't do that, and accepting that much data
 	 * can make us take up more cache space than we're supposed to,
 	 * so error. */
-	code = rx_Error(v->call);
+        static int warned;
+        if (!warned) {
+            warned = 1;
+            afs_warn("afs: Warning: FetchData64 returned more data than "
+                     "requested (requested %ld, got %ld); this should not "
+                     "happen! Aborting fetch request.\n",
+                     (long)size, (long)*alength);
+        }
 	RX_AFS_GUNLOCK();
-	code1 = rx_EndCall(v->call, code);
+	code = rx_EndCall(v->call, RX_PROTOCOL_ERROR);
 	RX_AFS_GLOCK();
 	v->call = NULL;
 	code = EIO;
     }
 
-    if (!code && code1)
-	code = code1;
-
     if (code) {
 	osi_FreeSmallSpace(v);
         return code;
@@ -1119,11 +1150,15 @@ afs_CacheFetchProc(struct afs_conn *tc, struct rx_connection *rxconn,
 
     adc->validPos = base;
 
-    if ( !code ) do {
+    if (code) {
+        goto done;
+    }
+
+    do {
 	if (avc->f.states & CForeign) {
 	    code = (*ops->more)(rock, &length, &moredata);
 	    if ( code )
-		break;
+		goto done;
 	}
 #ifndef AFS_NOSTATS
 	bytesToXfer += length;
@@ -1146,11 +1181,11 @@ afs_CacheFetchProc(struct afs_conn *tc, struct rx_connection *rxconn,
 			   ICL_TYPE_POINTER, avc, ICL_TYPE_INT32, code,
 			   ICL_TYPE_INT32, length);
 		code = -34;
-		break;
+		goto done;
 	    }
 	    code = (*ops->write)(rock, fP, offset, bytesread, &byteswritten);
 	    if ( code )
-		break;
+		goto done;
 	    offset += bytesread;
 	    base += bytesread;
 	    length -= bytesread;
@@ -1163,10 +1198,11 @@ afs_CacheFetchProc(struct afs_conn *tc, struct rx_connection *rxconn,
 	}
 	code = 0;
     } while (moredata);
+ done:
     if (!code)
 	code = (*ops->close)(rock, avc, adc, tsmall);
     if (ops)
-	(*ops->destroy)(&rock, code);
+	code = (*ops->destroy)(&rock, code);
 
 #ifndef AFS_NOSTATS
     FillStoreStats(code, AFS_STATS_FS_XFERIDX_FETCHDATA, xferStartTime,
