@@ -88,6 +88,9 @@
 #define SYS_NAME_ID_x86_darwin_170       535
 #define SYS_NAME_ID_amd64_darwin_170     536
 #define SYS_NAME_ID_arm_darwin_170       537
+#define SYS_NAME_ID_x86_darwin_180       538
+#define SYS_NAME_ID_amd64_darwin_180     539
+#define SYS_NAME_ID_arm_darwin_180       540
 
 #define SYS_NAME_ID_next_mach20		 601
 #define SYS_NAME_ID_next_mach30		 602
@@ -179,6 +182,7 @@
 #define SYS_NAME_ID_sparc_linux2	1700
 #define SYS_NAME_ID_sparc_linux22	1701
 #define SYS_NAME_ID_sparc_linux24	1702
+#define SYS_NAME_ID_sparc_linux26	1703
 
 #define SYS_NAME_ID_sparc64_linux2	1800
 #define SYS_NAME_ID_sparc64_linux22	1801
@@ -230,7 +234,10 @@
 #define SYS_NAME_ID_i386_fbsd_101      2151
 #define SYS_NAME_ID_i386_fbsd_102      2152
 #define SYS_NAME_ID_i386_fbsd_103      2153
+#define SYS_NAME_ID_i386_fbsd_104      2154
 #define SYS_NAME_ID_i386_fbsd_110      2140
+#define SYS_NAME_ID_i386_fbsd_111      2141
+#define SYS_NAME_ID_i386_fbsd_120      2160
 
 #define SYS_NAME_ID_ia64_linux2		2200
 #define SYS_NAME_ID_ia64_linux22	2201
@@ -278,6 +285,8 @@
 #define SYS_NAME_ID_macppc_nbsd50       2541
 #define SYS_NAME_ID_amd64_nbsd60        2542
 #define SYS_NAME_ID_i386_nbsd60         2543
+#define SYS_NAME_ID_amd64_nbsd70        2544
+#define SYS_NAME_ID_i386_nbsd70         2545
 
 #define SYS_NAME_ID_i386_obsd31		2600
 #define SYS_NAME_ID_i386_obsd32		2601
@@ -338,7 +347,10 @@
 #define SYS_NAME_ID_amd64_fbsd_101     3051
 #define SYS_NAME_ID_amd64_fbsd_102     3052
 #define SYS_NAME_ID_amd64_fbsd_103     3053
+#define SYS_NAME_ID_amd64_fbsd_104     3054
 #define SYS_NAME_ID_amd64_fbsd_110     3040
+#define SYS_NAME_ID_amd64_fbsd_111     3041
+#define SYS_NAME_ID_amd64_fbsd_120     3060
 
 #define SYS_NAME_ID_amd64_w2k           3400
 
@@ -376,29 +388,12 @@
 #define SYS_NAME_ID_amd64_obsd53        4022
 #define SYS_NAME_ID_amd64_obsd54        4023
 
-/*
- * Placeholder to keep system-wide standard flags since this file is included by all
- * files (i.e in afs/param.h)
- */
-#ifdef	notdef
-/* Should be enabled by src sites that are compiling afs in a kerberos environment
- * (i.e. use their headers and libs) and want to use the realm-related kerberos code
- */
-#define	AFS_ATHENA_STDENV		1
-#endif
-#ifdef	AFS_ATHENA_STDENV
-/* So that they don't get our own version of the kerb-related function  in libutil.a */
-#define	afs_krb_get_lrealm(r,n)	krb_get_lrealm(r,n)
-#define	AFS_REALM_SZ		REALM_SZ
-#endif
+#define SYS_NAME_ID_arm64_linux2	4100
+#define SYS_NAME_ID_arm64_linux26	4103
+
+#define	AFS_REALM_SZ	64
 
-/* Should be enabled by src sites that are compiling afs without the kerb headers/libs
- * but want to use the kerberos realm-related code
- */
-#define	AFS_KERBREALM_ENV	1
-#ifdef	AFS_KERBREALM_ENV
-#define	AFS_REALM_SZ		64
-#endif
 /* Specifies the number of equivalent local realm names */
 #define AFS_NUM_LREALMS 4
+
 #endif /* __AFS_SYSNAMES_INCL_ENV_ */
